"""
This file computes the chi squared test in code. Run the file to get the chi squared output values.
Later on I used excel for the chi squared analysis however because it was easier, but this script was used to
verify the paired comparison values.
"""

import numpy as np
from scipy.stats import chi2_contingency, chi2
import pandas as pd

def chi_squared_overall():
    """
    This method calculates the chi squared test for the overall difference between observations for the
    different simulation methods. The current frequency values are the ones obtained from the results,
    replace these with your own if you want to use this code.

    :return: chi2_results, a dictionary containing the different attainable values from the chi squared test for
    the overall aggregated data
    """

    # Observed frequencies
    observed = np.array([[56, 61], [6, 6], [19, 14]])

    # Calculate the total for each row and column
    row_totals = observed.sum(axis=1)
    col_totals = observed.sum(axis=0)
    grand_total = observed.sum()

    # Calculate expected frequencies
    expected = np.outer(row_totals, col_totals) / grand_total

    # Calculate chi-squared statistic
    chi2_stat = ((observed - expected) ** 2 / expected).sum()

    # Calculate p-value from chi-squared distribution with degrees of freedom (df = (rows-1)*(columns-1))
    df = (observed.shape[0] - 1) * (observed.shape[1] - 1)
    p_value = chi2.sf(chi2_stat, df)

    # Prepare the results
    chi2_results = {
        'chi2_stat': chi2_stat,
        'p-value': p_value,
        'degrees of freedom': df,
        'expected frequencies': expected
    }

    return chi2_results


def chi_squared_per_audio_type():
    """
    This method calculates the chi squared test values per audio sample type.
    The current frequency values are the ones obtained from the results,
    replace these with your own if you want to use this code.

    :return: chi2_results, a dictionary containing the different attainable values from the chi squared test for
    each audio sample type
    """

    # Data for each audio type
    classical_data = np.array([[20, 17], [2, 2], [5, 8]])
    sine_sweep_data = np.array([[12, 18], [4, 3], [11, 6]])
    speech_data = np.array([[24, 26], [0, 1], [3, 0]])

    # Perform chi-squared tests
    chi2_classical_stat, chi2_classical_p, chi2_classical_dof, chi2_classical_ex = chi2_contingency(classical_data)
    chi2_sine_sweep_stat, chi2_sine_sweep_p, chi2_sine_sweep_dof, chi2_sine_sweep_ex = chi2_contingency(sine_sweep_data)
    chi2_speech_stat, chi2_speech_p, chi2_speech_dof, chi2_speech_ex = chi2_contingency(speech_data)

    # Prepare the results
    chi2_results_audio_types = {
        'Classical': {
            'chi2_stat': chi2_classical_stat,
            'p-value': chi2_classical_p,
            'degrees of freedom': chi2_classical_dof,
            'expected frequencies': chi2_classical_ex
        },
        'Sine Sweep': {
            'chi2_stat': chi2_sine_sweep_stat,
            'p-value': chi2_sine_sweep_p,
            'degrees of freedom': chi2_sine_sweep_dof,
            'expected frequencies': chi2_sine_sweep_ex
        },
        'Speech': {
            'chi2_stat': chi2_speech_stat,
            'p-value': chi2_speech_p,
            'degrees of freedom': chi2_speech_dof,
            'expected frequencies': chi2_speech_ex
        }
    }


    results_df = pd.DataFrame(chi2_results_audio_types).T
    return results_df

print(chi_squared_overall())
print(chi_squared_per_audio_type())

