"""
This file was used for the convolution of the Room Impulse Responses with the anechoic audio segments. Calling the
method with the respective paths as arguments takes the audios and convolves them into an output path.
"""

import librosa
import soundfile as sf
from scipy.signal import convolve
import numpy as np


def load_audio(file_path, target_sr=16000):
    """
    This method loads the audio with librosa at a specific sample rate of 16000, as that was used for the speech.
    44100 Hz was used for the classical music segments, and was specified in the code for those.
    """
    audio, sr = librosa.load(file_path, sr=target_sr)
    return audio, sr

def normalize_audio(audio):
    """
    Normalizes the audio so that the SPL is the same for all audio snippets that the participants listen to.
    """
    return audio / np.max(np.abs(audio))

def convolve_audio(sample_path, rir_path, output_path):
    """
    Handles convolution logic, takes the input paths and output path, convolves and writes to output.
    """

    # We have the audio path and the RIRs path, load them
    audio_sample, sr_sample = load_audio(sample_path)
    impulse_response, sr_rir = load_audio(rir_path)

    # If the sample rates don't correspond, we give an error as we cant convolve them
    if sr_sample != sr_rir:
        raise ValueError("Sample rate of the audio sample and impulse response must match")

    # Convolve and normalize the output
    convolved_audio = convolve(audio_sample, impulse_response, mode='full')
    convolved_audio = normalize_audio(convolved_audio)

    sf.write(output_path, convolved_audio, sr_sample)
    print(f"Convolved audio saved to {output_path}")


def main():
    import argparse

    parser = argparse.ArgumentParser(description='Convolve an audio sample with a room impulse response.')
    parser.add_argument('--sample', type=str, help='Path to the audio sample file')
    parser.add_argument('--rir', type=str, help='Path to the room impulse response file')
    parser.add_argument('--output', type=str, help='Path to save the convolved audio file')

    args = parser.parse_args()
    convolve_audio(args.sample, args.rir, args.output)


if __name__ == "__main__":
    main()
