"""
Simulation of RIRs with Pyroomacoustics is done in this file.
"""

import numpy as np
import pyroomacoustics as pra
import matplotlib.pyplot as plt
import soundfile as sf

# Room dimensions (in meters)
room_dimensions = [8.1, 6.8, 3.07]

# Source and receiver positions (as speaker and microphone respectively)
speaker_position = [6.01, 2.019, 1.175]
microphone_position = [2.11, 3.33, 0.99]

# Desired reverberation time (RT60) in seconds
rt60 = 0.34

# Sample rate
fs = 48000

# Calculate the energy absorption and maximum order using Sabine's formula
e_absorption, max_order = pra.inverse_sabine(rt60, room_dimensions)


# Create the room with the calculated absorption and maximum order
room = pra.ShoeBox(
    room_dimensions,
    fs=fs,
    materials=pra.Material(e_absorption),
    max_order=3,
    ray_tracing=True
)
room.set_ray_tracing()

# Add the microphone array
room.add_microphone_array(
    pra.MicrophoneArray(np.array([microphone_position]).T, room.fs)
)

# Add the source (speaker) position
room.add_source(speaker_position)

# Compute the room impulse response
room.compute_rir()

# Plot the room impulse response for the first microphone
rir = room.rir[0][0]
# Make sure max value in the array is the first value
rir = rir[np.argmax(rir, axis=0):]

t60 = pra.measure_rt60(rir, fs=fs, plot=True)
print(t60)

plt.figure()
plt.plot(np.arange(len(rir)) / room.fs, rir)
plt.title('Room Impulse Response')
plt.xlabel('Time [s]')
plt.ylabel('Amplitude')
plt.grid()

# Save the RIR as a wav file and save the generated plot for visual inspection
plt.savefig('insert/path/here')
sf.write('insert/path/here/')

